//
// Copyright (c) 2022-Present, Okta, Inc. and/or its affiliates. All rights reserved.
// The Okta software accompanied by this notice is provided pursuant to the Apache License, Version 2.0 (the "License.")
//
// You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0.
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
// WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//
// See the License for the specific language governing permissions and limitations under the License.
//

import Foundation
import XCTest

class SignInScreen: Screen, WebLogin {
    let app: XCUIApplication
    let testCase: XCTestCase
    
    lazy var ephemeralSwitch = app.switches["ephemeral_switch"]
    lazy var signInButton = app.buttons["sign_in_button"]
    lazy var clientIdLabel = app.buttons["client_id_label"]

    init(_ testCase: XCTestCase, app: XCUIApplication = XCUIApplication()) {
        self.testCase = testCase
        self.app = app
    }
    func isVisible(timeout: TimeInterval = 3) {
        XCTAssertTrue(app.staticTexts["Okta Web Sign In"].waitForExistence(timeout: timeout))
        XCTAssertFalse(app.staticTexts["Not configured"].exists)
    }
    
    func validate(clientId: String) {
        XCTAssertEqual(clientIdLabel.label, clientId)
    }
}
